@@ -232,15 +232,29 @@ def _decode_type(
             "t",
         )
 
-    try:
-        if spec["t"]["data_enc"] == "b":
-            doc_dec["t"] = s[idx : idx + f_len].hex().upper()
-        else:
+    if spec["t"]["data_enc"] == "b":
+        doc_dec["t"] = s[idx : idx + f_len].hex().upper()
+    else:
+        try:
             doc_dec["t"] = s[idx : idx + f_len].decode(spec["t"]["data_enc"])
-    except Exception as e:
-        raise DecodeError(
-            f"Failed to decode ({e})", s, doc_dec, doc_enc, idx, "t"
-        ) from None
+        except LookupError:
+            raise DecodeError(
+                "Failed to decode field, unknown encoding specified",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                "t",
+            ) from None
+        except Exception:
+            raise DecodeError(
+                "Failed to decode field, invalid data",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                "t",
+            ) from None
 
     return idx + f_len
 
@@ -301,17 +315,42 @@ def _decode_bitmaps(
             "p",
         )
 
-    try:
-        if spec["p"]["data_enc"] == "b":
-            doc_dec["p"] = s[idx : idx + f_len].hex().upper()
-            bm = s[idx : idx + f_len]
-        else:
+    if spec["p"]["data_enc"] == "b":
+        doc_dec["p"] = s[idx : idx + f_len].hex().upper()
+        bm = s[idx : idx + f_len]
+    else:
+        try:
             doc_dec["p"] = s[idx : idx + f_len].decode(spec["p"]["data_enc"])
+        except LookupError:
+            raise DecodeError(
+                "Failed to decode field, unknown encoding specified",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                "p",
+            ) from None
+        except Exception:
+            raise DecodeError(
+                "Failed to decode field, invalid data",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                "p",
+            ) from None
+
+        try:
             bm = bytes.fromhex(doc_dec["p"])
-    except Exception as e:
-        raise DecodeError(
-            f"Failed to decode ({e})", s, doc_dec, doc_enc, idx, "p"
-        ) from None
+        except Exception:
+            raise DecodeError(
+                "Failed to decode field, non-hex data",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                "p",
+            ) from None
 
     fields.update(
         [
@@ -348,17 +387,42 @@ def _decode_bitmaps(
             "1",
         )
 
-    try:
-        if spec["1"]["data_enc"] == "b":
-            doc_dec["1"] = s[idx : idx + f_len].hex().upper()
-            bm = s[idx : idx + f_len]
-        else:
+    if spec["1"]["data_enc"] == "b":
+        doc_dec["1"] = s[idx : idx + f_len].hex().upper()
+        bm = s[idx : idx + f_len]
+    else:
+        try:
             doc_dec["1"] = s[idx : idx + f_len].decode(spec["1"]["data_enc"])
+        except LookupError:
+            raise DecodeError(
+                "Failed to decode field, unknown encoding specified",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                "1",
+            ) from None
+        except Exception:
+            raise DecodeError(
+                "Failed to decode field, invalid data",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                "1",
+            ) from None
+
+        try:
             bm = bytes.fromhex(doc_dec["1"])
-    except Exception as e:
-        raise DecodeError(
-            f"Failed to decode ({e})", s, doc_dec, doc_enc, idx, "1"
-        ) from None
+        except Exception:
+            raise DecodeError(
+                "Failed to decode field, non-hex data",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                "1",
+            ) from None
 
     fields.update(
         [
@@ -432,19 +496,55 @@ def _decode_field(
         enc_field_len: int = spec[field_key]["max_len"]
     # Variable field length
     else:
-        try:
-            # BCD length
-            if spec[field_key]["len_enc"] == "b":
+        # BCD length
+        if spec[field_key]["len_enc"] in {"b", "bcd"}:
+            try:
                 enc_field_len = int(s[idx : idx + len_type].hex(), 10)
-            # Text length
-            else:
-                enc_field_len = int(
-                    s[idx : idx + len_type].decode(spec[field_key]["len_enc"]), 10
+            except Exception:
+                raise DecodeError(
+                    "Failed to decode field length, invalid BCD data",
+                    s,
+                    doc_dec,
+                    doc_enc,
+                    idx,
+                    field_key,
+                ) from None
+        # Text length
+        else:
+            try:
+                decoded_length = s[idx : idx + len_type].decode(
+                    spec[field_key]["len_enc"]
                 )
-        except Exception as e:
-            raise DecodeError(
-                f"Failed to decode length ({e})", s, doc_dec, doc_enc, idx, field_key
-            ) from None
+            except LookupError:
+                raise DecodeError(
+                    "Failed to decode field length, unknown encoding specified",
+                    s,
+                    doc_dec,
+                    doc_enc,
+                    idx,
+                    field_key,
+                ) from None
+            except Exception:
+                raise DecodeError(
+                    "Failed to decode field length, invalid data",
+                    s,
+                    doc_dec,
+                    doc_enc,
+                    idx,
+                    field_key,
+                ) from None
+
+            try:
+                enc_field_len = int(decoded_length)
+            except Exception:
+                raise DecodeError(
+                    "Failed to decode field length, non-numeric data",
+                    s,
+                    doc_dec,
+                    doc_enc,
+                    idx,
+                    field_key,
+                ) from None
 
         if enc_field_len > spec[field_key]["max_len"]:
             raise DecodeError(
@@ -489,23 +589,35 @@ def _decode_field(
             field_key,
         )
 
-    try:
-        if spec[field_key]["data_enc"] == "b":
-            doc_dec[field_key] = doc_enc[field_key]["data"].hex().upper()
-            if len_count == "nibbles" and enc_field_len & 1:
-                doc_dec[field_key] = _remove_pad_field(
-                    s, idx, doc_dec, doc_enc, field_key, spec, enc_field_len
-                )
-        else:
+    if spec[field_key]["data_enc"] == "b":
+        doc_dec[field_key] = doc_enc[field_key]["data"].hex().upper()
+        if len_count == "nibbles" and enc_field_len & 1:
+            doc_dec[field_key] = _remove_pad_field(
+                s, idx, doc_dec, doc_enc, field_key, spec, enc_field_len
+            )
+    else:
+        try:
             doc_dec[field_key] = doc_enc[field_key]["data"].decode(
                 spec[field_key]["data_enc"]
             )
-    except DecodeError:
-        raise
-    except Exception as e:
-        raise DecodeError(
-            f"Failed to decode ({e})", s, doc_dec, doc_enc, idx, field_key
-        ) from None
+        except LookupError:
+            raise DecodeError(
+                "Failed to decode field, unknown encoding specified",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                field_key,
+            ) from None
+        except Exception:
+            raise DecodeError(
+                "Failed to decode field, invalid data",
+                s,
+                doc_dec,
+                doc_enc,
+                idx,
+                field_key,
+            ) from None
 
     return idx + byte_field_len
 
