@@ -400,7 +400,7 @@ def test_header_negative_incorrect_encoding():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .unknown encoding: invalid.: field h",
+        match="Failed to encode field, unknown encoding specified: field h",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -409,9 +409,6 @@ def test_header_negative_incorrect_ascii_data():
     """
     ASCII header is required by spec and provided.
     However, the data is not ASCII
-    CPython and PyPy throw differently worded exception
-    CPython: 'ascii' codec can't encode characters in position 0-5: ordinal not in range(128)
-    PyPy:    'ascii' codec can't encode character '\\xff' in position 0: ordinal not in range(128)
     """
     spec["h"]["data_enc"] = "ascii"
     spec["h"]["len_type"] = 0
@@ -426,7 +423,7 @@ def test_header_negative_incorrect_ascii_data():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .'ascii' codec can't encode character.*: ordinal not in range.128..: field h",
+        match="Failed to encode field, invalid data: field h",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -446,7 +443,7 @@ def test_header_negative_incorrect_bcd_data():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .non-hexadecimal number found in fromhex.. arg at position 0.: field h",
+        match="Failed to encode field, non-hex data: field h",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -628,7 +625,7 @@ def test_variable_header_bdc_over_max():
     BDC variable header is required and over max is provided
     """
     spec["h"]["data_enc"] = "b"
-    spec["h"]["len_enc"] = "b"
+    spec["h"]["len_enc"] = "bcd"
     spec["h"]["len_type"] = 2
     spec["h"]["max_len"] = 2
     spec["t"]["data_enc"] = "ascii"
@@ -646,12 +643,9 @@ def test_variable_header_bdc_over_max():
 def test_variable_header_bdc_odd():
     """
     BDC variable header is required and odd length is provided
-    CPython and PyPy throw differently worded exception
-    CPython: non-hexadecimal number found in fromhex() arg at position 5
-    PyPy:    non-hexadecimal number found in fromhex() arg at position 4
     """
     spec["h"]["data_enc"] = "b"
-    spec["h"]["len_enc"] = "b"
+    spec["h"]["len_enc"] = "bcd"
     spec["h"]["len_type"] = 2
     spec["h"]["max_len"] = 6
     spec["t"]["data_enc"] = "ascii"
@@ -661,7 +655,7 @@ def test_variable_header_bdc_odd():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .non-hexadecimal number found in fromhex.. arg at position 4|5.: field h",
+        match="Failed to encode field, odd-length hex data: field h",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -734,12 +728,21 @@ def test_variable_header_bdc_ebcdic_length():
     assert doc_dec.keys() == set(["h", "t", "p"])
 
 
-def test_variable_header_bcd_present():
+# fmt: off
+@pytest.mark.parametrize(
+    ["len_enc"],
+    [
+        ("b",),
+        ("bcd",),
+    ],
+)
+# fmt: on
+def test_variable_header_bcd_present(len_enc: str):
     """
     BCD variable header is required and provided
     """
     spec["h"]["data_enc"] = "b"
-    spec["h"]["len_enc"] = "b"
+    spec["h"]["len_enc"] = len_enc
     spec["h"]["len_type"] = 2
     spec["h"]["max_len"] = 6
     spec["t"]["data_enc"] = "ascii"
@@ -767,12 +770,21 @@ def test_variable_header_bcd_present():
     assert doc_dec.keys() == set(["h", "t", "p"])
 
 
-def test_variable_header_bcd_present_zero_length():
+# fmt: off
+@pytest.mark.parametrize(
+    ["len_enc"],
+    [
+        ("b",),
+        ("bcd",),
+    ],
+)
+# fmt: on
+def test_variable_header_bcd_present_zero_length(len_enc: str):
     """
     BCD zero-length variable header is required and provided
     """
     spec["h"]["data_enc"] = "b"
-    spec["h"]["len_enc"] = "b"
+    spec["h"]["len_enc"] = len_enc
     spec["h"]["len_type"] = 2
     spec["h"]["max_len"] = 6
     spec["t"]["data_enc"] = "ascii"
@@ -816,7 +828,7 @@ def test_variable_header_incorrect_encoding():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode length .unknown encoding: invalid.: field h",
+        match="Failed to encode field length, unknown encoding specified: field h",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -896,10 +908,7 @@ def test_type_ascii_over_max():
 def test_type_ascii_incorrect_data():
     """
     ASCII message type is required and provided.
-    However, the data is not ASCII
-    CPython and PyPy throw differently worded exception
-    CPython: 'ascii' codec can't encode characters in position 0-3: ordinal not in range(128)
-    PyPy:    'ascii' codec can't encode character '\\xff' in position 0: ordinal not in range(128)
+    However, the data is not ASCII.
     """
     spec["h"]["data_enc"] = "ascii"
     spec["h"]["len_type"] = 0
@@ -914,7 +923,7 @@ def test_type_ascii_incorrect_data():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .'ascii' codec can't encode character.*: ordinal not in range.128..: field t",
+        match="Failed to encode field, invalid data: field t",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -1094,9 +1103,6 @@ def test_type_bdc_over_max():
 def test_type_bdc_odd():
     """
     BDC message type is required and odd length is provided
-    CPython and PyPy throw differently worded exception
-    CPython: non-hexadecimal number found in fromhex() arg at position 3
-    PyPy:    non-hexadecimal number found in fromhex() arg at position 2
     """
     spec["h"]["data_enc"] = "ascii"
     spec["h"]["len_type"] = 0
@@ -1108,7 +1114,7 @@ def test_type_bdc_odd():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .non-hexadecimal number found in fromhex.. arg at position 2|3.: field t",
+        match="Failed to encode field, odd-length hex data: field t",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -1128,7 +1134,7 @@ def test_type_bdc_non_hex():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .non-hexadecimal number found in fromhex.. arg at position 3.: field t",
+        match="Failed to encode field, non-hex data: field t",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -1180,7 +1186,7 @@ def test_type_incorrect_encoding():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .unknown encoding: invalid.: field t",
+        match="Failed to encode field, unknown encoding specified: field t",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -1341,7 +1347,7 @@ def test_primary_bitmap_incorrect_encoding():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .unknown encoding: invalid.: field p",
+        match="Failed to encode field, unknown encoding specified: field p",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -1363,7 +1369,7 @@ def test_secondary_bitmap_incorrect_encoding():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .unknown encoding: invalid.: field 1",
+        match="ailed to encode field, unknown encoding specified: field 1",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -1630,9 +1636,6 @@ def test_fixed_field_ascii_incorrect_data():
     """
     ASCII fixed field is required and provided.
     However, the data is not ASCII
-    CPython and PyPy throw differently worded exception
-    CPython: 'ascii' codec can't encode characters in position 0-1: ordinal not in range(128)
-    PyPy:    'ascii' codec can't encode character '\\xff' in position 0: ordinal not in range(128)
     """
     spec["h"]["data_enc"] = "ascii"
     spec["h"]["len_type"] = 0
@@ -1651,7 +1654,7 @@ def test_fixed_field_ascii_incorrect_data():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .'ascii' codec can't encode character.*: ordinal not in range.128..: field 2",
+        match="Failed to encode field, invalid data: field 2",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -1943,9 +1946,6 @@ def test_fixed_field_bdc_over_max():
 def test_fixed_field_bdc_odd():
     """
     BDC fixed field is required and odd length is provided
-    CPython and PyPy throw differently worded exception
-    CPython: non-hexadecimal number found in fromhex() arg at position 5
-    PyPy:    non-hexadecimal number found in fromhex() arg at position 4
     """
     spec["h"]["data_enc"] = "ascii"
     spec["h"]["len_type"] = 0
@@ -1960,7 +1960,7 @@ def test_fixed_field_bdc_odd():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .non-hexadecimal number found in fromhex.. arg at position 4|5.: field 2",
+        match="Failed to encode field, odd-length hex data: field 2",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -1983,7 +1983,7 @@ def test_fixed_field_bdc_non_hex():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .non-hexadecimal number found in fromhex.. arg at position 2.: field 2",
+        match="Failed to encode field, non-hex data: field 2",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -2085,7 +2085,7 @@ def test_fixed_field_incorrect_encoding():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .unknown encoding: invalid.: field 2",
+        match="Failed to encode field, unknown encoding specified: field 2",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -2308,7 +2308,7 @@ def test_variable_field_bdc_over_max():
     spec["2"]["len_type"] = 2
     spec["2"]["max_len"] = 5
     spec["2"]["data_enc"] = "b"
-    spec["2"]["len_enc"] = "b"
+    spec["2"]["len_enc"] = "bcd"
 
     doc_dec = {"h": "header", "t": "0210", "2": "123456789012"}
 
@@ -2322,9 +2322,6 @@ def test_variable_field_bdc_over_max():
 def test_variable_field_bdc_odd():
     """
     BDC variable field is required and odd length is provided
-    CPython and PyPy throw differently worded exception
-    CPython: non-hexadecimal number found in fromhex() arg at position 5
-    PyPy:    non-hexadecimal number found in fromhex() arg at position 4
     """
     spec["h"]["data_enc"] = "ascii"
     spec["h"]["len_type"] = 0
@@ -2334,13 +2331,13 @@ def test_variable_field_bdc_odd():
     spec["2"]["len_type"] = 2
     spec["2"]["max_len"] = 10
     spec["2"]["data_enc"] = "b"
-    spec["2"]["len_enc"] = "b"
+    spec["2"]["len_enc"] = "bcd"
 
     doc_dec = {"h": "header", "t": "0210", "2": "12345"}
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .non-hexadecimal number found in fromhex.. arg at position 4|5.: field 2",
+        match="Failed to encode field, odd-length hex data: field 2",
     ):
         iso8583.encode(doc_dec, spec=spec)
 
@@ -2427,7 +2424,16 @@ def test_variable_field_bdc_ebcdic_length():
     assert doc_dec.keys() == set(["h", "t", "p", "2"])
 
 
-def test_variable_field_bcd_present():
+# fmt: off
+@pytest.mark.parametrize(
+    ["len_enc"],
+    [
+        ("b",),
+        ("bcd",),
+    ],
+)
+# fmt: on
+def test_variable_field_bcd_present(len_enc: str) -> None:
     """
     BCD variable field is required and provided
     """
@@ -2439,7 +2445,7 @@ def test_variable_field_bcd_present():
     spec["2"]["len_type"] = 2
     spec["2"]["max_len"] = 10
     spec["2"]["data_enc"] = "b"
-    spec["2"]["len_enc"] = "b"
+    spec["2"]["len_enc"] = len_enc
 
     doc_dec = {"h": "header", "t": "0210", "2": "1122"}
 
@@ -2467,7 +2473,16 @@ def test_variable_field_bcd_present():
     assert doc_dec.keys() == set(["h", "t", "p", "2"])
 
 
-def test_variable_field_bcd_present_zero_length():
+# fmt: off
+@pytest.mark.parametrize(
+    ["len_enc"],
+    [
+        ("b",),
+        ("bcd",),
+    ],
+)
+# fmt: on
+def test_variable_field_bcd_present_zero_length(len_enc: str):
     """
     BCD zero-length variable field is required and provided
     """
@@ -2479,7 +2494,7 @@ def test_variable_field_bcd_present_zero_length():
     spec["2"]["len_type"] = 2
     spec["2"]["max_len"] = 10
     spec["2"]["data_enc"] = "b"
-    spec["2"]["len_enc"] = "b"
+    spec["2"]["len_enc"] = len_enc
 
     doc_dec = {"h": "header", "t": "0210", "2": ""}
 
@@ -2526,6 +2541,6 @@ def test_variable_field_incorrect_encoding():
 
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode length .unknown encoding: invalid.: field 2",
+        match="Failed to encode field length, unknown encoding specified: field 2",
     ):
         iso8583.encode(doc_dec, spec=spec)
