@@ -13,12 +13,12 @@ import pytest
     [
         ("ascii", "ascii", 2, 8, "bytes",   b"02004000000000000000041234", b"04"),
         ("ascii", "ascii", 2, 8, "nibbles", b"02004000000000000000081234", b"08"),
-        ("ascii", "b",     2, 8, "bytes",   b"02004000000000000000\x00\x041234", b"\x00\x04"),
-        ("ascii", "b",     2, 8, "nibbles", b"02004000000000000000\x00\x081234", b"\x00\x08"),
+        ("ascii", "bcd",   2, 8, "bytes",   b"02004000000000000000\x00\x041234", b"\x00\x04"),
+        ("ascii", "bcd",   2, 8, "nibbles", b"02004000000000000000\x00\x081234", b"\x00\x08"),
         ("b",     "ascii", 2, 8, "bytes",   b"0200400000000000000002\x12\x34", b"02"),
         ("b",     "ascii", 2, 8, "nibbles", b"0200400000000000000004\x12\x34", b"04"),
-        ("b",     "b",     2, 8, "bytes",   b"02004000000000000000\x00\x02\x12\x34", b"\x00\x02"),
-        ("b",     "b",     2, 8, "nibbles", b"02004000000000000000\x00\x04\x12\x34", b"\x00\x04"),
+        ("b",     "bcd",   2, 8, "bytes",   b"02004000000000000000\x00\x02\x12\x34", b"\x00\x02"),
+        ("b",     "bcd",   2, 8, "nibbles", b"02004000000000000000\x00\x04\x12\x34", b"\x00\x04"),
         ("ascii", "ascii", 0, 4, "bytes",   b"020040000000000000001234", b""),
         ("ascii", "ascii", 0, 8, "nibbles", b"020040000000000000001234", b""),
         ("b",     "ascii", 0, 2, "bytes",   b"02004000000000000000\x12\x34", b""),
@@ -58,16 +58,16 @@ def test_encode_nibbles(
     ["len_enc", "len_type", "max_len", "len_count", "pad", "result", "result_f2_len"],
     [
         ("ascii", 2, 8, "nibbles", "0", b"0200400000000000000003\x01\x23", b"03"),
-        ("b",     2, 8, "nibbles", "0", b"02004000000000000000\x00\x03\x01\x23", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "0", b"02004000000000000000\x00\x03\x01\x23", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "0", b"02004000000000000000\x01\x23", b""),
         ("ascii", 2, 8, "nibbles", "F", b"0200400000000000000003\xF1\x23", b"03"),
-        ("b",     2, 8, "nibbles", "F", b"02004000000000000000\x00\x03\xF1\x23", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "F", b"02004000000000000000\x00\x03\xF1\x23", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "F", b"02004000000000000000\xF1\x23", b""),
         ("ascii", 2, 8, "nibbles", "01", b"0200400000000000000003\x01\x23", b"03"),
-        ("b",     2, 8, "nibbles", "01", b"02004000000000000000\x00\x03\x01\x23", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "01", b"02004000000000000000\x00\x03\x01\x23", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "01", b"02004000000000000000\x01\x23", b""),
         ("ascii", 2, 8, "nibbles", "F1", b"0200400000000000000003\xF1\x23", b"03"),
-        ("b",     2, 8, "nibbles", "F1", b"02004000000000000000\x00\x03\xF1\x23", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "F1", b"02004000000000000000\x00\x03\xF1\x23", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "F1", b"02004000000000000000\xF1\x23", b""),
     ],
 )
@@ -105,16 +105,16 @@ def test_encode_nibbles_odd_left_pad(
     ["len_enc", "len_type", "max_len", "len_count", "pad", "result", "result_f2_len"],
     [
         ("ascii", 2, 8, "nibbles", "0", b"0200400000000000000003\x12\x30", b"03"),
-        ("b",     2, 8, "nibbles", "0", b"02004000000000000000\x00\x03\x12\x30", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "0", b"02004000000000000000\x00\x03\x12\x30", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "0", b"02004000000000000000\x12\x30", b""),
         ("ascii", 2, 8, "nibbles", "F", b"0200400000000000000003\x12\x3F", b"03"),
-        ("b",     2, 8, "nibbles", "F", b"02004000000000000000\x00\x03\x12\x3F", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "F", b"02004000000000000000\x00\x03\x12\x3F", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "F", b"02004000000000000000\x12\x3F", b""),
         ("ascii", 2, 8, "nibbles", "01", b"0200400000000000000003\x12\x30", b"03"),
-        ("b",     2, 8, "nibbles", "01", b"02004000000000000000\x00\x03\x12\x30", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "01", b"02004000000000000000\x00\x03\x12\x30", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "01", b"02004000000000000000\x12\x30", b""),
         ("ascii", 2, 8, "nibbles", "F1", b"0200400000000000000003\x12\x3F", b"03"),
-        ("b",     2, 8, "nibbles", "F1", b"02004000000000000000\x00\x03\x12\x3F", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "F1", b"02004000000000000000\x00\x03\x12\x3F", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "F1", b"02004000000000000000\x12\x3F", b""),
     ],
 )
@@ -153,7 +153,7 @@ def test_encode_nibbles_odd_no_pad() -> None:
     spec["p"]["data_enc"] = "ascii"
 
     spec["2"]["data_enc"] = "b"
-    spec["2"]["len_enc"] = "b"
+    spec["2"]["len_enc"] = "bcd"
     spec["2"]["len_type"] = 2
     spec["2"]["max_len"] = 8
     spec["2"]["len_count"] = "nibbles"
@@ -161,7 +161,27 @@ def test_encode_nibbles_odd_no_pad() -> None:
     decoded = {"t": "0200", "2": "1"}
     with pytest.raises(
         iso8583.EncodeError,
-        match="Failed to encode .*: field 2",
+        match="Failed to encode field, odd-length nibble data, specify pad: field 2",
+    ):
+        iso8583.encode(decoded, spec=spec)
+
+
+def test_encode_nibbles_non_hex() -> None:
+    spec = copy.deepcopy(iso8583.specs.default)
+    spec["t"]["data_enc"] = "ascii"
+    spec["p"]["data_enc"] = "ascii"
+
+    spec["2"]["data_enc"] = "b"
+    spec["2"]["len_enc"] = "bcd"
+    spec["2"]["len_type"] = 2
+    spec["2"]["max_len"] = 8
+    spec["2"]["len_count"] = "nibbles"
+    spec["2"]["right_pad"] = "f"
+
+    decoded = {"t": "0200", "2": "x"}
+    with pytest.raises(
+        iso8583.EncodeError,
+        match="Failed to encode field, non-hex data: field 2",
     ):
         iso8583.encode(decoded, spec=spec)
 
@@ -172,12 +192,12 @@ def test_encode_nibbles_odd_no_pad() -> None:
     [
         ("ascii", "ascii", 2, 8, "bytes",   b"02004000000000000000041234", b"04"),
         ("ascii", "ascii", 2, 8, "nibbles", b"02004000000000000000081234", b"08"),
-        ("ascii", "b",     2, 8, "bytes",   b"02004000000000000000\x00\x041234", b"\x00\x04"),
-        ("ascii", "b",     2, 8, "nibbles", b"02004000000000000000\x00\x081234", b"\x00\x08"),
+        ("ascii", "bcd",   2, 8, "bytes",   b"02004000000000000000\x00\x041234", b"\x00\x04"),
+        ("ascii", "bcd",   2, 8, "nibbles", b"02004000000000000000\x00\x081234", b"\x00\x08"),
         ("b",     "ascii", 2, 8, "bytes",   b"0200400000000000000002\x12\x34", b"02"),
         ("b",     "ascii", 2, 8, "nibbles", b"0200400000000000000004\x12\x34", b"04"),
-        ("b",     "b",     2, 8, "bytes",   b"02004000000000000000\x00\x02\x12\x34", b"\x00\x02"),
-        ("b",     "b",     2, 8, "nibbles", b"02004000000000000000\x00\x04\x12\x34", b"\x00\x04"),
+        ("b",     "bcd",   2, 8, "bytes",   b"02004000000000000000\x00\x02\x12\x34", b"\x00\x02"),
+        ("b",     "bcd",   2, 8, "nibbles", b"02004000000000000000\x00\x04\x12\x34", b"\x00\x04"),
         ("ascii", "ascii", 0, 4, "bytes",   b"020040000000000000001234", b""),
         ("ascii", "ascii", 0, 8, "nibbles", b"020040000000000000001234", b""),
         ("b",     "ascii", 0, 2, "bytes",   b"02004000000000000000\x12\x34", b""),
@@ -215,16 +235,16 @@ def test_decode_nibbles(
     ["len_enc", "len_type", "max_len", "len_count", "pad", "data", "result_f2_len"],
     [
         ("ascii", 2, 8, "nibbles", "0", b"0200400000000000000003\x01\x23", b"03"),
-        ("b",     2, 8, "nibbles", "0", b"02004000000000000000\x00\x03\x01\x23", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "0", b"02004000000000000000\x00\x03\x01\x23", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "0", b"02004000000000000000\x01\x23", b""),
         ("ascii", 2, 8, "nibbles", "F", b"0200400000000000000003\xF1\x23", b"03"),
-        ("b",     2, 8, "nibbles", "F", b"02004000000000000000\x00\x03\xF1\x23", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "F", b"02004000000000000000\x00\x03\xF1\x23", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "F", b"02004000000000000000\xF1\x23", b""),
         ("ascii", 2, 8, "nibbles", "01", b"0200400000000000000003\x01\x23", b"03"),
-        ("b",     2, 8, "nibbles", "01", b"02004000000000000000\x00\x03\x01\x23", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "01", b"02004000000000000000\x00\x03\x01\x23", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "01", b"02004000000000000000\x01\x23", b""),
         ("ascii", 2, 8, "nibbles", "F1", b"0200400000000000000003\xF1\x23", b"03"),
-        ("b",     2, 8, "nibbles", "F1", b"02004000000000000000\x00\x03\xF1\x23", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "F1", b"02004000000000000000\x00\x03\xF1\x23", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "F1", b"02004000000000000000\xF1\x23", b""),
     ],
 )
@@ -260,16 +280,16 @@ def test_decode_nibbles_left_pad(
     ["len_enc", "len_type", "max_len", "len_count", "pad", "data", "result_f2_len"],
     [
         ("ascii", 2, 8, "nibbles", "0", b"0200400000000000000003\x12\x30", b"03"),
-        ("b",     2, 8, "nibbles", "0", b"02004000000000000000\x00\x03\x12\x30", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "0", b"02004000000000000000\x00\x03\x12\x30", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "0", b"02004000000000000000\x12\x30", b""),
         ("ascii", 2, 8, "nibbles", "F", b"0200400000000000000003\x12\x3F", b"03"),
-        ("b",     2, 8, "nibbles", "F", b"02004000000000000000\x00\x03\x12\x3F", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "F", b"02004000000000000000\x00\x03\x12\x3F", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "F", b"02004000000000000000\x12\x3F", b""),
         ("ascii", 2, 8, "nibbles", "01", b"0200400000000000000003\x12\x30", b"03"),
-        ("b",     2, 8, "nibbles", "01", b"02004000000000000000\x00\x03\x12\x30", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "01", b"02004000000000000000\x00\x03\x12\x30", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "01", b"02004000000000000000\x12\x30", b""),
         ("ascii", 2, 8, "nibbles", "F1", b"0200400000000000000003\x12\x3F", b"03"),
-        ("b",     2, 8, "nibbles", "F1", b"02004000000000000000\x00\x03\x12\x3F", b"\x00\x03"),
+        ("bcd",   2, 8, "nibbles", "F1", b"02004000000000000000\x00\x03\x12\x3F", b"\x00\x03"),
         ("ascii", 0, 3, "nibbles", "F1", b"02004000000000000000\x12\x3F", b""),
     ],
 )
@@ -306,7 +326,7 @@ def test_decode_nibbles_odd_no_pad() -> None:
     spec["p"]["data_enc"] = "ascii"
 
     spec["2"]["data_enc"] = "b"
-    spec["2"]["len_enc"] = "b"
+    spec["2"]["len_enc"] = "bcd"
     spec["2"]["len_type"] = 2
     spec["2"]["max_len"] = 8
     spec["2"]["len_count"] = "nibbles"
